import { parse, stringify } from 'wellknown';
import circle from '@turf/circle';

export const loadImage = (url) => new Promise((imageLoaded) => {
  const image = new Image();
  image.onload = () => imageLoaded(image);
  image.src = url;
});

const canvasTintImage = (image, color) => {
  const canvas = document.createElement('canvas');
  canvas.width = image.width * devicePixelRatio;
  canvas.height = image.height * devicePixelRatio;
  canvas.style.width = `${image.width}px`;
  canvas.style.height = `${image.height}px`;

  const context = canvas.getContext('2d');

  context.save();
  context.fillStyle = color;
  context.globalAlpha = 1;
  context.fillRect(0, 0, canvas.width, canvas.height);
  context.globalCompositeOperation = 'destination-atop';
  context.globalAlpha = 1;
  context.drawImage(image, 0, 0, canvas.width, canvas.height);
  context.restore();

  return canvas;
};

export const prepareIcon = (background, icon, color) => {
  const canvas = document.createElement('canvas');
  canvas.width = background.width * devicePixelRatio;
  canvas.height = background.height * devicePixelRatio;
  canvas.style.width = `${background.width}px`;
  canvas.style.height = `${background.height}px`;

  const context = canvas.getContext('2d');
  context.drawImage(background, 0, 0, canvas.width, canvas.height);

  if (icon) {
    const iconRatio = 0.5;
    const imageWidth = canvas.width * iconRatio;
    const imageHeight = canvas.height * iconRatio;
    context.drawImage(canvasTintImage(icon, color), (canvas.width - imageWidth) / 2, (canvas.height - imageHeight) / 2, imageWidth, imageHeight);
  }

  return context.getImageData(0, 0, canvas.width, canvas.height);
};

export const reverseCoordinates = (it) => {
  if (!it) {
    return it;
  } if (Array.isArray(it)) {
    if (it.length === 2 && typeof it[0] === 'number' && typeof it[1] === 'number') {
      return [it[1], it[0]];
    }
    return it.map((it) => reverseCoordinates(it));
  }
  return {
    ...it,
    coordinates: reverseCoordinates(it.coordinates),
  };
};

export const geofenceToFeature = (theme, item) => {
  let geometry;
  if (item.area.indexOf('CIRCLE') > -1) {
    const coordinates = item.area.replace(/CIRCLE|\(|\)|,/g, ' ').trim().split(/ +/);
    const options = { steps: 32, units: 'meters' };
    const polygon = circle([Number(coordinates[1]), Number(coordinates[0])], Number(coordinates[2]), options);
    geometry = polygon.geometry;
  } else {
    geometry = reverseCoordinates(parse(item.area));
  }
  return {
    id: item.id,
    type: 'Feature',
    geometry,
    properties: {
      name: item.name,
      color: item.attributes.color || theme.palette.geometry.main,
    },
  };
};

export const geometryToArea = (geometry) => stringify(reverseCoordinates(geometry));

export const findFonts = (map) => {
  const { glyphs } = map.getStyle();
  if (glyphs.startsWith('https://tiles.openfreemap.org')) {
    return ['Noto Sans Regular'];
  }
  return ['Open Sans Regular', 'Arial Unicode MS Regular'];
};

export const fontSize = (markerSize) => {
 console.log('markerSize:');
 console.log(markerSize);
  return 1;
};

export const drawGeofenceBoundingBox = (map,geofences,selectedGeofenceId) => {

  let attrs = geofences[selectedGeofenceId].attributes;
  if(attrs.pickUpPoints){
    let bBoxPtArr = JSON.parse(geofences[selectedGeofenceId].attributes.boundingBox);
    let coordinates = [];
    for(var i=0;i<bBoxPtArr.length;i++){
      let c=bBoxPtArr[i];
      console.log(c);
      coordinates.push([c.lon,c.lat]);
    }
    let bBoxId = 'bBox'+selectedGeofenceId;
    console.log(coordinates);

    
    if(map.getLayer(bBoxId)){
      map.removeLayer(bBoxId);
    }
  
    if(map.getSource(bBoxId+'source')){
      map.removeSource(bBoxId+'source');
    }


      map.addSource(bBoxId+'source', {
          'type': 'geojson',
          'data': {
              'type': 'Feature',
              'geometry': {
                  'type': 'Polygon',
                  'coordinates': [
                      coordinates
                  ]
              }
          }
      });
      map.addLayer({
          'id': bBoxId,
          'type': 'fill',
          'source': bBoxId+'source',
          'layout': {},
          'paint': {
              'fill-color': '#C8A4D4',
              'fill-opacity': 0.3
          }
      });

  }
}


export const drawPickUpPoints = async(map,geofences,selectedGeofenceId) => {
  let attrs = geofences[selectedGeofenceId].attributes;
  let image = await map.loadImage('/target.png');
  console.log(image);
  if(attrs.pickUpPoints && image!=null){
    let pickUpPtArr = JSON.parse(attrs.pickUpPoints);
    console.log(pickUpPtArr);
    for(var i=0;i<pickUpPtArr.length;i++){
      let ithPP = pickUpPtArr[i];
      let c=ithPP.coordinate.split(",");
      let ppName = ithPP.name + selectedGeofenceId.toString();
      console.log('name:'+ppName);

      if(map.getLayer(ppName+'Layer')){
        map.removeLayer(ppName+'Layer');
      }
    
      if(map.getSource(ppName+'point')){
        map.removeSource(ppName+'point');
      }

      if(map.getImage(ppName)){
        map.removeImage(ppName);
      }
      
        map.addImage(ppName, image.data);
      console.log(c);
      let point = [ parseFloat(c[1].trim()) , parseFloat(c[0].trim()) ];

      map.addSource(ppName+'point', {
                'type': 'geojson',
                'data': {
                    'type': 'FeatureCollection',
                    'features': [
                        {
                            'type': 'Feature',
                            'geometry': {
                                'type': 'Point',
                                'coordinates': point
                            }
                        }
                    ]
                }
          });
        
            map.addLayer({
                'id': ppName+'Layer',
                'type': 'symbol',
                'source': ppName+'point',
                'layout': {
                    'icon-image': ppName,
                    'icon-size': 1.0,
                    'text-field': ithPP.name,
                    'text-font': findFonts(map),
                    'text-size': 16,
                    'text-offset': [0, -2],
                }
            });
    }
  }
}



